local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
local GUI = HeadCount:NewModule("GUI")
if not HeadCount then return end

local L = LibStub("AceLocale-3.0"):GetLocale("HeadCount2")
local AGU = LibStub("AceGUI-3.0")
local MainWindow = nil


local SimpleGroupBackdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
}

function GUI:ShowLootPopup(lootId,raidId)
end

local function ShowTooltip(widget)
	GameTooltip:SetOwner(widget.frame)
	GameTooltip:AddLine(widget:GetUserData("tooltip.key"))
	GameTooltip:Show()
end
local function HideTooltip()
	GameTooltip:Hide()
end

local function SetupTooltip(widget,tooltip)
	widget:SetUserData("tooltip.key",tooltip)
	widget:SetCallback("OnEnter",ShowTooltip)
	widget:SetCallback("OnLeave",HideTooltip)
end

function GUI:Refresh()
	self:UpdateRaidHistoryLabel()
	if self.annouce_window then
		AGU:Release(self.annouce_window)
	end
	self:PopulateRaidList()
end

function GUI:UpdateRaidHistoryLabel()
	if MainWindow then
		local raidCount = HeadCount.db.profile.raidListWrapper.numberOfRaids
		MainWindow.history_label:SetText(string.format(L["gui.history.title"],raidCount))
	end
end

local function RemoveAllRaids()
	HeadCount:GetModule("RaidTracker"):removeAllRaids()
	GUI:Refresh()
end

local function EndActiveRaid()
	local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
	local activityTime = HeadCount:NewInstance("Time",utcDateTimeInSeconds)
	HeadCount:GetModule("RaidTracker"):endCurrentRaid(activityTime)
end

local function ShowExportWindow(data)
	local f = AGU:Create("Frame")
	local control = AGU:Create("MultiLineEditBox")
	f:ReleaseChildren()
	f:SetCallback("OnClose", function(widget, event) control.button:Show(); AGU:Release(widget) end)
	f:SetTitle(L["gui.export.title"])
	f:SetLayout("fill")
	--f = scroll
	control:SetHeight(300)
	control:SetText(data)
	control.width = "fill"
	f:AddChild(control)
	f:PerformLayout()
	f:SetWidth(600)
	f:SetHeight(300)
	f:Show()
	control.button:Hide()
	if not IsMacClient() then
		-- Windows or Linux
		f.statustext:SetText(L["gui.win.export.tooltip"])
	else
		f.statustext:SetText(L["gui.mac.export.tooltip"])
	end
	f.frame:SetFrameStrata("TOOLTIP")
end

function GUI:ShowPlayersInRaid()
	if GUI.selectedRaid and MainWindow then
		local raid = GUI.selectedRaid
		MainWindow.lower_label:SetText("Player List")
		GUI.LowerBox:ReleaseChildren()
		local sort = AGU:Create("HeadCountHeaderItem")
		sort:SetColumn(1,"","",30,"")
		sort:SetColumn(2,"Name","Name",100,"Name tooltip")
		sort:SetColumn(3,"Start","Start",150,"start tooltip")
		sort:SetColumn(4,"End","End",150,"end tooltip")
		sort:SetColumn(5,"Total","Total",130,"total tooltip")
		sort:SetCallback("SortChosen",function(widget,event,colName)
			MainWindow.memberSort = colName
			MainWindow.sortDir = not MainWindow.sortDir
			GUI:ShowPlayersInRaid()
		end)
		sort:SetRelativeWidth(1)
		GUI.LowerBox:AddChild(sort)
		local numberOfPlayers = raid:GetNumberOfPlayers()
		local orderedPlayerList = raid:retrieveOrderedPlayerList(HeadCount:getRAID_MEMBER_SORT()[MainWindow.memberSort], true, true, true, false, false, MainWindow.sortDir)
		for k=1,#orderedPlayerList do
			local player = orderedPlayerList[k]
			local idLabel =  AGU:Create("Label")
			idLabel:SetText("    "..k)
			idLabel:SetWidth(40)
			GUI.LowerBox:AddChild(idLabel)

			local nLabel = AGU:Create("Label")
			nLabel:SetText(player.name)
			nLabel:SetWidth(100)
			GUI.LowerBox:AddChild(nLabel)

			local sT = AGU:Create("Label")
			sT:SetText(HeadCount:getDateTimeAsString(player:retrieveStartingTime()))
			sT:SetWidth(150)
			GUI.LowerBox:AddChild(sT)

			local eT = AGU:Create("Label")
			eT:SetText(HeadCount:getDateTimeAsString(player:retrieveEndingTime()))
			eT:SetWidth(150)
			GUI.LowerBox:AddChild(eT)

			local tT = AGU:Create("Label")
			tT:SetText(HeadCount:getSecondsAsString(player:getTotalTime()))
			tT:SetWidth(130)
			GUI.LowerBox:AddChild(tT)
		end
	end
end

function GUI:ShowBossListInRaid()
	if GUI.selectedRaid and MainWindow then
		local raid = GUI.selectedRaid
		MainWindow.lower_label:SetText("Boss List")
		GUI.LowerBox:ReleaseChildren()
	end
end

function GUI:ShowLootInRaid()
	if GUI.selectedRaid and MainWindow then
		local raid = GUI.selectedRaid
		MainWindow.lower_label:SetText("Loot List")
		GUI.LowerBox:ReleaseChildren()
	end
end

function GUI:ShowWaitListInRaid()
	if GUI.selectedRaid and MainWindow then
		local raid = GUI.selectedRaid
		MainWindow.lower_label:SetText("Wait List")
		GUI.LowerBox:ReleaseChildren()
	end
end

function GUI:PopulateRaidList(initial)
	if MainWindow and self.UpperBox then
		local rlW = HeadCount:GetModule("RaidTracker").raidListWrapper
		self.UpperBox:ReleaseChildren()
		local orderedRaidList = rlW:retrieveOrderedRaidList(false)
		for k=1,#orderedRaidList do
			local v = orderedRaidList[k]
			-- TODO Make a Widget to show a Given Row so I can do Highlighting and Such
			local zone = v.zone
			local difficulty = v.difficulty
			local difficultyString = ""
			if difficulty then
				difficultyString = _G["RAID_DIFFICULTY"..difficulty]
			end
			if zone then
				if difficulty then
					zone = zone .. " (" .. difficultyString .. ")"
				else
					zone = zone
				end
			end
			-- Add active label to zone
			if not v.isFinalized and HeadCount:GetModule("RaidTracker").isCurrentRaidActive then
				-- raid is not finalized and raid is active
				if zone then
					zone = zone .. " (" .. L["Active"] .. ")"
				else
					zone = "(" .. L["Active"] .. ")"
				end
			end
			if not zone then
				zone = UNKNOWN
			end
			local raidInfoItem = AGU:Create("HeadCountRaidItem")
			raidInfoItem:SetFullWidth(true)
			raidInfoItem:SetNumber(k)
			raidInfoItem:SetDate(HeadCount:getDateTimeAsString(v:retrieveStartingTime()))
			raidInfoItem:SetZone(zone)
			raidInfoItem:SetRaidID(v:retrieveStartingTime().epoc)
			raidInfoItem.export.tt = L["gui.export.raid"]
			raidInfoItem.delete.tt = L["gui.remove.raid"]

			raidInfoItem:SetCallback("RaidDeleted", function (widget, event, id)
				HeadCount:GetModule("RaidTracker"):removeRaid(id)
				GUI:Refresh()
			end)

			raidInfoItem:SetCallback("RaidExported", function (widget, event, id)
				local raid = HeadCount:GetModule("RaidTracker").raidListWrapper:getRaidById(id)
				local data = HeadCount:exportRaid(raid)
				ShowExportWindow(data)
			end)

			raidInfoItem:SetCallback("RaidSelected", function(widget,event,id, row)
				for k,v in ipairs(GUI.UpperBox.children) do
					v:SetSelected(false)
					if k == row then
						v:SetSelected(true)
					end
				end
				local raid = HeadCount:GetModule("RaidTracker").raidListWrapper:getRaidById(id)
				GUI.selectedRaid = raid
				GUI:ShowPlayersInRaid(raid)
			end)
			if initial and k == 1 then
				GUI.selectedRaid = v
				raidInfoItem:SetSelected(true)
			end
			self.UpperBox:AddChild(raidInfoItem)
		end
		if initial then
			self:ShowPlayersInRaid()
		end
	end
end

function GUI:ShowTracker()
	if MainWindow then return end
	MainWindow = AGU:Create("Window")
	MainWindow:SetTitle("HeadCount2 Tracker")
	MainWindow:SetWidth(620)
	MainWindow:SetHeight(420)
	MainWindow:SetCallback("OnClose", function(widget)
		widget:ReleaseChildren()
		widget:Release()
		MainWindow = nil
	end)
	--MainWindow.statustext:GetParent():Hide()
	MainWindow:SetLayout("Flow")
	MainWindow:EnableResize(false)
	-- Create upper Box
	-- UpperLabel
	local raidCount = HeadCount.db.profile.raidListWrapper.numberOfRaids
	local raidTitle = AGU:Create("Label")
	raidTitle:SetText(string.format(L["gui.history.title"],raidCount))
	raidTitle:SetColor(0.63,0.63,1.0,1.0)
	raidTitle:SetRelativeWidth(.90)
	MainWindow:AddChild(raidTitle)
	MainWindow.history_label = raidTitle

	-- Annouce Icon
	local raidAnnounce = AGU:Create("InteractiveLabel")
	raidAnnounce:SetImage("Interface\\Buttons\\UI-GuildButton-MOTD-Up")
	raidAnnounce:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	raidAnnounce:SetImageSize(13,13)
	raidAnnounce:SetRelativeWidth(0.03)
	SetupTooltip(raidAnnounce,L["gui.announce.wait.list"])
	raidAnnounce:SetCallback("OnClick", GUI.ShowAnnounceWindow)
	MainWindow:AddChild(raidAnnounce)

	-- End Raid
	local raidEnd = AGU:Create("InteractiveLabel")
	raidEnd:SetImage("Interface\\Buttons\\UI-GroupLoot-Pass-Up")
	raidEnd:SetHighlight("Interface\\Buttons\\UI-GroupLoot-Pass-Highlight")
	raidEnd:SetImageSize(13,13)
	raidEnd:SetRelativeWidth(0.03)
	SetupTooltip(raidEnd,L["gui.end.active.raid"])
	raidEnd:SetCallback("OnClick", function()
		local currentRaidId = HeadCount:GetModule("RaidTracker").raidListWrapper:retrieveMostRecentRaidId()
		if HeadCount:GetModule("RaidTracker"):isRaidActive() then
			GUI:ConfirmChoice(L["gui.confirm.end.raid"],EndActiveRaid)
		else
			HeadCount:LogInformation(string.format(L["info.end.raid.noraidsexists"], HeadCount.TITLE, HeadCount.VERSION))
		end
	end)
	MainWindow:AddChild(raidEnd)

	-- Delete all raids
	local raidWipe = AGU:Create("InteractiveLabel")
	raidWipe:SetImage("Interface\\CharacterFrame\\Disconnect-Icon")
	raidWipe:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	raidWipe:SetImageSize(24,24)
	raidWipe:SetRelativeWidth(0.03)
	SetupTooltip(raidWipe,L["gui.remove.all.raids"])
	raidWipe:SetCallback("OnClick", function()
		if HeadCount.db.profile.raidListWrapper.numberOfRaids > 0 then
			GUI:ConfirmChoice(L["gui.confirm.remove.all"],RemoveAllRaids)
		end
	end)
	MainWindow:AddChild(raidWipe)

	-- Upper Box simple groupwith our border
	local UpperBox = AGU:Create("SimpleGroup")
	UpperBox:SetHeight(150)
	UpperBox:SetFullWidth(true)
	UpperBox:SetLayout("Fill")

	local UBScroll = AGU:Create("ScrollFrame")

	UpperBox:AddChild(UBScroll)

	local border = CreateFrame("Frame", nil, UpperBox.content)
	border:SetPoint("TOPLEFT", 0, 0)
	border:SetPoint("BOTTOMRIGHT", -1, 3)
	border:SetBackdrop(SimpleGroupBackdrop)
	border:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	border:SetBackdropBorderColor(0.4, 0.4, 0.4)
	UBScroll:SetHeight(150)
	MainWindow:AddChild(UpperBox)
	UBScroll:SetLayout("Flow")
	GUI.UpperBox = UBScroll

	-- Create Lower Box
	local lowerTitle = AGU:Create("Label")
	lowerTitle:SetText("")
	lowerTitle:SetColor(0.63,0.63,1.0,1.0)
	lowerTitle:SetRelativeWidth(.87)
	MainWindow:AddChild(lowerTitle)
	MainWindow.lower_label = lowerTitle

	local memberButton = AGU:Create("InteractiveLabel")
	memberButton:SetImage("Interface\\CharacterFrame\\UI-StateIcon")
	memberButton.image:SetTexCoord(0.5,1.0,0.0,0.5)
	memberButton:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	memberButton:SetImageSize(18,18)
	memberButton:SetRelativeWidth(0.03)
	SetupTooltip(memberButton,L["gui.player.list"])
	memberButton:SetCallback("OnClick",GUI.ShowPlayersInRaid)
	MainWindow.memberSort = "Start"
	MainWindow:AddChild(memberButton)

	local waitlistButton = AGU:Create("InteractiveLabel")
	waitlistButton:SetImage("Interface\\GossipFrame\\PetitionGossipIcon")
	waitlistButton:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	waitlistButton:SetImageSize(13,13)
	waitlistButton:SetRelativeWidth(0.03)
	SetupTooltip(waitlistButton,L["gui.wait.list"])
	waitlistButton:SetCallback("OnClick", GUI.ShowWaitListInRaid)
	MainWindow:AddChild(waitlistButton)

	local bossButton = AGU:Create("InteractiveLabel")
	bossButton:SetImage("Interface\\TargetingFrame\\UI-TargetingFrame-Skull")
	bossButton:SetHighlight("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
	bossButton:SetImageSize(14,11)
	bossButton:SetRelativeWidth(0.03)
	SetupTooltip(bossButton,L["gui.boss.list"])
	bossButton:SetCallback("OnClick", GUI.ShowBossListInRaid)
	MainWindow:AddChild(bossButton)

	local lootButton = AGU:Create("InteractiveLabel")
	lootButton:SetImage("Interface\\Buttons\\UI-GroupLoot-Coin-Up")
	lootButton:SetHighlight("Interface\\Buttons\\UI-GroupLoot-Coin-Highlight")
	lootButton:SetImageSize(14,14)
	lootButton:SetRelativeWidth(0.03)
	SetupTooltip(lootButton,L["gui.loot.list"])
	lootButton:SetCallback("OnClick", GUI.ShowLootInRaid)
	MainWindow:AddChild(lootButton)

	-- Make Lower Label

	local LowerBox = AGU:Create("SimpleGroup")
	LowerBox:SetHeight(165)
	LowerBox:SetFullWidth(true)
	LowerBox:SetLayout("Fill")

	local LBScroll = AGU:Create("ScrollFrame")
	LBScroll:SetHeight(135)
	LowerBox:AddChild(LBScroll)

	local lborder = CreateFrame("Frame", nil, LowerBox.content)
	lborder:SetPoint("TOPLEFT", 0, 5)
	lborder:SetPoint("BOTTOMRIGHT", 3, -5)
	lborder:SetBackdrop(SimpleGroupBackdrop)
	lborder:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	lborder:SetBackdropBorderColor(0.4, 0.4, 0.4)
	LBScroll:SetLayout("Flow")
	GUI.LowerBox = LBScroll

	MainWindow:AddChild(LowerBox)
	MainWindow.sortDir = true
	GUI:PopulateRaidList(true)

end

function GUI:ConfirmChoice(message,callback)
	if not StaticPopupDialogs["HEADCOUNT_CONFIRM"] then
		StaticPopupDialogs["HEADCOUNT_CONFIRM"] = {
			text = "",
			button1 = ACCEPT,
			button2 = CANCEL,
			OnAccept = function()
				return false
			end,
			timeout = 0,
			whileDead = 1,
			hideOnEscape = 1,
			exclusive= true,
		}
	end
	StaticPopupDialogs["HEADCOUNT_CONFIRM"].text = message
	StaticPopupDialogs["HEADCOUNT_CONFIRM"].OnAccept = callback
	StaticPopup_Show("HEADCOUNT_CONFIRM")
end

function GUI:ShowAnnounceWindow()
	if GUI.annouce_window then return end
	self = GUI
	local frame = AGU:Create("HeadCountDialog")
	frame:SetTitle(L["gui.annouce.title"])
	frame:SetWidth(400)
	frame:SetHeight(200)
	frame:SetCallback("OnClose", function(widget)
		widget:ReleaseChildren()
		widget:Release()
		GUI.annouce_window = nil
	end)

	frame:SetCallback("OnAccept", function(widget)
		local choice = frame:GetUserData("type-choice")
		local channel = frame:GetUserData("channel-choice")
		widget:ReleaseChildren()
		widget:Release()
		GUI.annouce_window = nil
		if choice == 1 then
			HeadCount:GetModule("RaidTracker"):announceWaitlist(channel)
		else
			HeadCount:GetModule("RaidTracker"):processListWaitlist(channel)
		end
	end)

	frame:SetAcceptLabel("Annouce")
	frame:SetCancelLabel("Cancel")
	frame:EnableHotKey()

	frame:SetLayout("Flow")
	self.annouce_window = frame
	frame:SetUserData("type-choice",1)
	frame:SetUserData("channel-choice","GUILD")

	local spacer1 = AGU:Create("Label")
	spacer1:SetRelativeWidth(0.22)
	local spacer2 = AGU:Create("Label")
	spacer2:SetRelativeWidth(0.22)
	local spacer3 = AGU:Create("Label")
	spacer3:SetRelativeWidth(0.2)

	-- Create first DropDown
	local methodDrop = AGU:Create("Dropdown")
	methodDrop:SetLabel("Type")
	methodDrop:SetList( { [1] = L["gui.announce.wait.list"], [2] = L["gui.list.wait.list"]})
	methodDrop:SetCallback("OnValueChanged", function(self,event, key,checked)
		frame:SetUserData("type-choice",key)
	end)
	methodDrop:SetValue(1)
	frame:AddChild(spacer1)
	frame:AddChild(methodDrop)
	frame:AddChild(spacer3)
	methodDrop.label:SetTextColor(0.63,0.63,1,1)
	-- Second Dropdown
	local channelDrop = AGU:Create("Dropdown")
	channelDrop:SetLabel("Channel")
	channelDrop:SetList(HeadCount.CHANNEL_LIST)
	channelDrop:SetCallback("OnValueChanged", function(self, event, key,checked)
		frame:SetUserData("channel-choice",key)
	end)
	channelDrop:SetValue("GUILD")
	channelDrop.label:SetTextColor(0.63,0.63,1,1)
	frame:AddChild(spacer2)
	frame:AddChild(channelDrop)
end